@@ -1,7 +1,8 @@
 import numpy as np
 
-from pyinterpolate.transform.prepare_kriging_data import prepare_kriging_data
 from pyinterpolate.distance.calculate_distances import calc_point_to_point_distance
+from pyinterpolate.transform.prepare_kriging_data import prepare_kriging_data
+from pyinterpolate.transform.tests import does_variogram_exist
 
 
 class Krige:
@@ -16,10 +17,10 @@ class Krige:
     - ordinary_kriging - ordinary kriging of unknown point value,
     - simple_kriging - simple kriging of unknown point value.
 
-    Class methods may raise ValueError if estimated value is below 0. You may use try: ... except: ... statement to overwrite those values
-        with some constant or NaN or you could use different semivariogram model. Sometimes this problem is related to
-        the input data, especially to the clustered groups of points. In this case aggregate those clusters and then estimate
-        semivariogram and perform kriging.
+    Class methods may raise ValueError if estimated value is below 0. You may use try: ... except: ... statement to
+        overwrite those values with some constant or NaN or you could use different semivariogram model. Sometimes this
+        problem is related to the input data, especially to the clustered groups of points. In this case aggregate
+        those clusters and then estimate semivariogram and perform kriging.
 
     INITLIALIZATION PARAMS:
 
@@ -33,11 +34,15 @@ class Krige:
         INPUT:
 
         :param semivariogram_model: (TheoreticalSemivariogram) Theoretical Semivariogram used for data interpolation,
-        :param known_points: (numpy array) dataset with known values and locations. Each column should represent different dimension and the last column represents values
-        example: [[dim_x1, dim_y1, val_1], [dim_x2, dim_y2, val_2]]
+        :param known_points: (numpy array) dataset with known values and locations. Each column should represent
+            different dimension and the last column represents values:
+            [[dim_x1, dim_y1, val_1], [dim_x2, dim_y2, val_2]]
         """
 
         self.dataset = known_points
+
+        # Test semivariogram model
+        does_variogram_exist(semivariogram_model)
         self.model = semivariogram_model
         self.distances = None
 
@@ -99,7 +104,7 @@ class Krige:
             sigma = np.sqrt(sigmasq)
         return zhat, sigma, w[-1], w
 
-    def simple_kriging(self, unknown_location, number_of_neighbours, mu=None, test_anomalies=True):
+    def simple_kriging(self, unknown_location, number_of_neighbours, global_mean, test_anomalies=True):
         """
         Function predicts value at unknown location with Simple Kriging technique.
 
@@ -108,9 +113,7 @@ class Krige:
         :param unknown_location: (tuple) position of unknown location,
         :param number_of_neighbours: (int) number of the closest locations to the unknown position which should be
             included in the modeling,
-        :param mu: (float) global mean which should be known before processing. If not given then it is calculated
-            from the sample but then it may cause a relative large errors (this mean is expectation of the random field,
-            so without knowledge of the ongoing processes it is unknown),
+        :param global_mean: (float) global mean which should be known before processing,
         :param test_anomalies: (bool) check if weights are negative.
 
         OUTPUT:
@@ -124,11 +127,6 @@ class Krige:
                                              number_of_neighbours=number_of_neighbours)
         n = number_of_neighbours
 
-        if mu is None:
-            vals = self.dataset[:, -1]
-            mu = np.sum(vals)
-            mu = mu / len(vals)
-
         unknown_distances = prepared_data[:, -1]
         k = self.model.predict(unknown_distances)
         k = k.T
@@ -138,9 +136,9 @@ class Krige:
         predicted = np.array(predicted_weights.reshape(n, n))
 
         w = np.linalg.solve(predicted, k)
-        r = prepared_data[:, -2] - mu
+        r = prepared_data[:, -2] - global_mean
         zhat = r.dot(w)
-        zhat = zhat + mu
+        zhat = zhat + global_mean
 
         # Test for anomalies
         if test_anomalies:
@@ -158,4 +156,4 @@ class Krige:
             sigma = 0
         else:
             sigma = np.sqrt(sigmasq)
-        return zhat, sigma, mu, w
+        return zhat, sigma, global_mean, w
